#include <LiquidCrystal.h>

int buttons[3] = {5, 6, 7};
int leds[3] = {8, 9, 10};
int nIO = 3;

int sequence[128];
int nSequence;

int isPlaying;
LiquidCrystal lcd(12, 11, 4, 3, 2, 13);
void setup() {
  // Setez pinii de output
  for (int i = 0; i < nIO; i++) {
    pinMode(leds[i], OUTPUT);
  }
  // Setez pinii de input
  for (int i = 0; i < nIO; i++) {
    pinMode(buttons[i], INPUT);
  }
  
  // IsPlaying e 0 pentru ca nu a inceput jocul
  isPlaying = 0;
  // Iau seed-ul de pe pinul 0, nefolosit
  randomSeed(analogRead(0));
  
  // Initializare LCD
  lcd.begin(16, 2);
  // Interfeta seriala
  Serial.begin(9600);
}

void loop() {
  
  if (isPlaying) {
    // Afisez secventa si asteapta sa fie replicata de catre utilizator
    lcd.clear();
    lcd.print("Showing Sequence"); 
    lcd.setCursor(0, 1);
    lcd.print("Level ");
    lcd.print(nSequence + 1);
    setSequence();
    lcd.clear();
    lcd.print("Getting Sequence"); 
    lcd.setCursor(0, 1);
    lcd.print("Level ");
    lcd.print(nSequence);
    getSequence();
    delay(1000);
  } else {
    // Astept o actiune din partea utilizatorului
    lcd.clear();
    lcd.print("Press a button"); 
    lcd.setCursor(0, 1);
    lcd.print("to start!");
    idleWaiting();
  }
}

void setSequence() {
  // Sting toate butoanele
  for (int pin = 0; pin < nIO; pin++) {
    digitalWrite(leds[pin], LOW);
  }
  // Pun o culoare noua la secventa
  sequence[nSequence] = random(0,nIO);
  nSequence++;
  
  // Afisez secventa pe interfata seriala si vizual folosind leduriile
  Serial.print("Sequence:");
  for (int i = 0; i < nSequence; i++) {
  Serial.print(" ");
  Serial.print(sequence[i]);
  }
  Serial.println(); 
  for (int i = 0; i < nSequence; i++) {
     delay(500);
      digitalWrite(leds[sequence[i]], HIGH);
     delay(500);
     digitalWrite(leds[sequence[i]], LOW);
  }
}

void getSequence() {
  // Pentru fiecare culoare din secventa verific daca a fost apasat butonul corect
  for (int i = 0; i < nSequence; i++) {
    // Jucatorul are un timp de 3 secunde pentru a apasa corect butonul corespunzator
    int index = getPressedButton(3000);
    // Afisez la ce buton s-a apasat, daca este cazul
    if (index == -1) {
      Serial.println("No button has been pressed!");
    } else {
      Serial.print("Pressed: ");
      Serial.print(index);
      Serial.print(", Required: ");
      Serial.print(sequence[i]);
      Serial.println();
    }

    // Daca butonul apasat este diferit de cel corespunzator ledului curent, sau daca nu s-a apasat nici un buton (-1)
    if (index != sequence[i]) {
      // Afisez un feedback catre utilizator
      lcd.clear();
      lcd.print("Game Over!"); 
      lcd.setCursor(0, 1);
      lcd.print("Level ");
      lcd.print(nSequence);
      for (int pin = 0; pin < nIO; pin++) {
        digitalWrite(leds[pin], HIGH);
      }
      delay(2000);
      for (int pin = 0; pin < nIO; pin++) {
        digitalWrite(leds[pin], LOW);
      }
      delay(1000);
      // Voi termina jocul curent
      isPlaying = 0;
      return;
    }
  }
  lcd.clear();
  lcd.print("Correct!"); 
}

void idleWaiting() {
  // Pentru fiecare culoare o voi afisa pentru o jumatate de secunda, ca sa fac un efect de valuri cu ledul,
  // sa se stie ca se asteapta un input pentru a incepe jocul
  for(int pin = 0; pin < nIO; pin++) {
    digitalWrite(leds[pin], HIGH);
    int buttonPressed = getPressedButton(500);
    digitalWrite(leds[pin], LOW);
    // Daca a fost apasat un buton
    if (buttonPressed != -1) {
      // Astept o secunda, dupa care incep jocul
      lcd.clear();
      lcd.print("Game starting..."); 
      delay(1000);
      nSequence = 0;
    isPlaying = 1;
    break;
    }
  }
}


int getPressedButton(int timeout) {
  long currentMillis = millis();
  long lastMillis = currentMillis;

  // Am facut un sistem de debouncing pentru a evita sa se citeasca ca un buton e declansat de mai multe ori cand este apasat
  while (currentMillis - lastMillis < timeout) {
  // Verific daca un buton s-a apasat momentan
    for (int pin = 0; pin < nIO; pin++) {
      // Daca un buton e apasat
      if (digitalRead(buttons[pin]) == HIGH) {
        // Aprind ledul corespunzator pentru feedback
        digitalWrite(leds[pin], HIGH);
        // Astept ca butonul sa nu mai fie apasat
        while (currentMillis - lastMillis < timeout) {
          // Daca butonul respectiv a fost lasat liber
          if (digitalRead(buttons[pin]) == LOW) {
            // Sting ledul si returnez valoarea
            digitalWrite(leds[pin], LOW);
            return pin;
          }
          currentMillis = millis();
        }

        // Daca nu s-a lasat liber butonul returnez ca nu am apasat nimic
        digitalWrite(leds[pin], LOW);
        return -1;
      }
    }
    currentMillis = millis();
  }
  // Daca nu s-a apasat nici un buton
  return -1;
}
